@@ -36,14 +36,13 @@ imgur_upload_job (ScreenshooterJob *job, GArray *param_values, GError **error)
   guchar *online_file_name = NULL;
   guchar *delete_hash = NULL;
   const gchar *proxy_uri;
-  SoupURI *soup_proxy_uri;
+  GUri *soup_proxy_uri;
 #ifdef DEBUG
   SoupLogger *log;
 #endif
-  guint status;
   SoupSession *session;
   SoupMessage *msg;
-  SoupBuffer *buf;
+  GBytes *buf, *response;
   GMappedFile *mapping;
   SoupMultipart *mp;
   xmlDoc *doc;
@@ -70,7 +69,7 @@ imgur_upload_job (ScreenshooterJob *job, GArray *param_values, GError **error)
 
   session = soup_session_new ();
 #ifdef DEBUG
-  log = soup_logger_new (SOUP_LOGGER_LOG_HEADERS, -1);
+  log = soup_logger_new (SOUP_LOGGER_LOG_HEADERS);
   soup_session_add_feature (session, (SoupSessionFeature *)log);
 #endif
 
@@ -79,53 +78,49 @@ imgur_upload_job (ScreenshooterJob *job, GArray *param_values, GError **error)
 
   if (proxy_uri != NULL)
     {
-      soup_proxy_uri = soup_uri_new (proxy_uri);
+      soup_proxy_uri = g_uri_parse (proxy_uri, G_URI_FLAGS_NONE, NULL);
       g_object_set (session, "proxy-uri", soup_proxy_uri, NULL);
-      soup_uri_free (soup_proxy_uri);
+      g_uri_unref (soup_proxy_uri);
     }
 
-  mapping = g_mapped_file_new(image_path, FALSE, NULL);
+  mapping = g_mapped_file_new (image_path, FALSE, NULL);
   if (!mapping) {
     g_object_unref (session);
-
     return FALSE;
   }
 
   mp = soup_multipart_new(SOUP_FORM_MIME_TYPE_MULTIPART);
-  buf = soup_buffer_new_with_owner (g_mapped_file_get_contents (mapping),
-                                    g_mapped_file_get_length (mapping),
-                                    mapping, (GDestroyNotify)g_mapped_file_unref);
+  buf = g_mapped_file_get_bytes (mapping);
 
   soup_multipart_append_form_file (mp, "image", NULL, NULL, buf);
   soup_multipart_append_form_string (mp, "name", title);
   soup_multipart_append_form_string (mp, "title", title);
-  msg = soup_form_request_new_from_multipart (upload_url, mp);
+  msg = soup_message_new_from_multipart (upload_url, mp);
 
   /* for v3 API - key registered *only* for xfce4-screenshooter! */
-  soup_message_headers_append (msg->request_headers, "Authorization", "Client-ID 66ab680b597e293");
+  soup_message_headers_append (soup_message_get_request_headers (msg), "Authorization", "Client-ID 66ab680b597e293");
   exo_job_info_message (EXO_JOB (job), _("Upload the screenshot..."));
-  status = soup_session_send_message (session, msg);
 
-  if (!SOUP_STATUS_IS_SUCCESSFUL (status))
+  response = soup_session_send_and_read (session, msg, NULL, &tmp_error);
+
+  g_mapped_file_unref (mapping);
+  g_bytes_unref (buf);
+  g_object_unref (session);
+  g_object_unref (msg);
+#ifdef DEBUG
+  g_object_unref (log);
+#endif
+
+  if (!response)
     {
-      TRACE ("Error during the POST exchange: %d %s\n",
-             status, msg->reason_phrase);
-
-      tmp_error = g_error_new (SOUP_HTTP_ERROR,
-                         status,
-                         _("An error occurred while transferring the data"
-                           " to imgur."));
+      TRACE ("Error during the POST exchange: %s\n", tmp_error->message);
       g_propagate_error (error, tmp_error);
-      g_object_unref (session);
-      g_object_unref (msg);
-
       return FALSE;
     }
 
-  TRACE("response was %s\n", msg->response_body->data);
+  TRACE("response was %s\n", (gchar*) g_bytes_get_data (response, NULL));
   /* returned XML is like <data type="array" success="1" status="200"><id>xxxxxx</id> */
-  doc = xmlParseMemory(msg->response_body->data,
-                                  strlen(msg->response_body->data));
+  doc = xmlParseMemory (g_bytes_get_data (response, NULL), g_bytes_get_size (response));
 
   root_node = xmlDocGetRootElement(doc);
   for (child_node = root_node->children; child_node; child_node = child_node->next)
@@ -135,16 +130,18 @@ imgur_upload_job (ScreenshooterJob *job, GArray *param_values, GError **error)
     else if (xmlStrEqual (child_node->name, (const xmlChar *) "deletehash"))
       delete_hash = xmlNodeGetContent (child_node);
   }
+
   TRACE("found picture id %s\n", online_file_name);
   xmlFreeDoc(doc);
-  soup_buffer_free (buf);
-  g_object_unref (session);
-  g_object_unref (msg);
+  g_bytes_unref (response);
 
   screenshooter_job_image_uploaded (job,
                                     (const gchar*) online_file_name,
                                     (const gchar*) delete_hash);
 
+  g_free (online_file_name);
+  g_free (delete_hash);
+
   return TRUE;
 }
 
